/*
*	robotomay2.c
*
*	Demonstrate programming the Microchip PIC32 processor, particularly the
*	one found on the ChipKit MAX32 board, PIC32MX795F512L.
*	(c) Dennis Clark, TTT Enterprises, March 2012
*
*	This program is REALLY open source, do what you want to do with it.
*	If you mention my name, that's cool too.  So are microbrew 
*	stouts and bitters.
*/
#include <plib.h>	// Include all API's, we then don't have to look them up.

/*
* 	Configure the system clocks
* 	We start with the MAX32 8MHz external crystal, multiply that by 20,
*	divide that by 2 and leave it at that: 80MHz, full speed.
*	POSCMOD = HS (high speed crystal), FNOSC = PRIPLL (Use the PLL)
*	Peripheral Bus (PB),10MHz rate, which makes for easy math.
*/
#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FPLLODIV = DIV_1
#pragma config POSCMOD = HS, FNOSC = PRIPLL 
#pragma config FPBDIV = DIV_8
// Not using the watchdog just yet.
#pragma config FWDTEN = OFF

#define SYS_FREQ (80000000L)

volatile long t_1ms = 0;

void __ISR(_TIMER_1_VECTOR, ipl2) Timer1Handler(void)
{
    // clear the interrupt flag
    mT1ClearIntFlag();
	t_1ms++;
}

void Init(void)
/*
 *	Set up all the things we want to play with.
 */
{
    unsigned long int	c1,c2,c3,cPort,cScan;
    CloseADC10();	// make sure it's off.
    
#define _MANUAL_ADC_

#ifdef _MANUAL_ADC_
    /*
    *	Configure the ADC system to use AN0 and AN1, which is A0/A1 on the Max32 board.
    * config1: 16bit unsigned 16 bit, auto trigger, sample when last conversion done
    * config2: Use AVCC/AVSS reference, Don't do a calibration, don't scan channels
    * config3: Use RC for Tad clock, use 15 Tad sample periods to aquire the sample.
    * configPort: AN0 and AN1
    * configScan: off
    * This setup sets everything up as a manual ADC conversion where you need to 
    * select manually the channel you want sampled, when you want to read it.
    */
    c1 = ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON;
    c2 = ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF;
    c3 = ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15;
    cPort = ENABLE_AN0_ANA | ENABLE_AN1_ANA;
    cScan = SKIP_SCAN_ALL;
#else
    /*
    *	Configure the ADC system to use AN0, which is A0 on the Max32 board.
    * config1: 16bit unsigned 16 bit, auto trigger, sample when last conversion done
    * config2: Use AVCC/AVSS reference, Don't do a calibration, scan channels per int.
    * config3: Use RC for Tad clock, use 15 Tad sample periods to aquire the sample.
    * configPort: AN0 and AN1
    * configScan: on, scan AN0 and AN1 on every pass.
    * This setup scans the configured ADC channels and stores their results in 
    * ADC1BUF0 and ADC1BUF1.
    */
    c1 = ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON;
    c2 = ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2;
    c3 = ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15;
    cPort = ENABLE_AN0_ANA | ENABLE_AN1_ANA;
    cScan = 0xFFFC;					// OpenADC10 inverts these "skips" them
#endif   
	SetChanADC10(0<<16);			// Not needed for scan, that always uses MUXA.
    OpenADC10(c1,c2,c3,cPort,cScan);
    EnableADC10();					// Go!
    
	/*
	*	Configure the system for its best performance, this is complicated, so
	*	lets let the compiler and system handle it!
	*	Then, we'll set the peripheral clock to 10MHz, which makes simple math...
	*/
	SYSTEMConfigPerformance(SYS_FREQ);
	mOSCSetPBDIV(OSC_PB_DIV_8);
	
	/*
	* 	Configure Timer 1 to interrupt every 1ms so we can increment
	*	a background ticker that we'll call t_1ms.  We'll give the 
	*	Timer1 interrupt a priority 2 setting, which is one step above
	*	the lowest interrupt priority.  We'll used vectored interrupts.
	*/
	OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_1, 10000);
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_2);
    INTEnableSystemMultiVectoredInt();
	
	// Set LED4 just to show we got this far
	PORTSetPinsDigitalOut(IOPORT_A,BIT_3);
}

void BlinkLED(unsigned short rate)
/*
*	Blink "LED4" on the Max32 board by toggling it every
*	"rate" milliseconds.
*/
{
	static delay = 0;
	
	if (t_1ms > delay) {
		mPORTAToggleBits(BIT_3);
		delay = t_1ms + rate;
	}
}	
	
int main(void)
{
	unsigned short value;	
	int alt = 0;
	
	Init();
	
	while (1) {
#ifdef _MANUAL_ADC_
		/*
		*	0 will select AN0, 1 will select AN1
		*/
		SetChanADC10(0<<16);
#endif
		/*
		*	In "manual" mode, 0 depends upon which
		*	channel we selected above.  In "scan"
		*	mode, 0 is ADC1BUF0 = AN0 and 
		*	1 is ADC1BUF1 = AN1.  No need to select
		*	a channel, both are stored for use.
		*/
		value = ReadADC10(0);
		BlinkLED(value);
	}
}
